@@ -23,17 +23,29 @@
  */
 
 #include "kaboutdata.h"
+#include "kpluginmetadata.h"
 
 #include <QCoreApplication>
-#include "qstandardpaths.h"
+#include <QStandardPaths>
 #include <QtCore/QFile>
 #include <QtCore/QTextStream>
 #include <QtCore/QSharedData>
 #include <QtCore/QVariant>
 #include <QtCore/QList>
+#include <QUrl>
 #include <QHash>
 #include <QCommandLineParser>
 #include <QCommandLineOption>
+#include <QJsonObject>
+#include <QLoggingCategory>
+
+#include <algorithm>
+
+
+Q_DECLARE_LOGGING_CATEGORY(KABOUTDATA)
+// logging category for this framework, default: log stuff >= warning
+Q_LOGGING_CATEGORY(KABOUTDATA, "kf5.kcoreaddons.kaboutdata", QtWarningMsg)
+
 
 class KAboutPerson::Private
 {
@@ -107,6 +119,17 @@ KAboutPerson &KAboutPerson::operator=(const KAboutPerson &other)
     return *this;
 }
 
+KAboutPerson KAboutPerson::fromJSON(const QJsonObject &obj)
+{
+    const QString name = KPluginMetaData::readTranslatedString(obj, QStringLiteral("Name"));
+    const QString task = KPluginMetaData::readTranslatedString(obj, QStringLiteral("Task"));
+    const QString email = obj[QStringLiteral("Email")].toString();
+    const QString website = obj[QStringLiteral("Website")].toString();
+    const QString userName = obj[QStringLiteral("UserName")].toString();
+    return KAboutPerson(name, task, email, website, userName);
+}
+
+
 class KAboutLicense::Private : public QSharedData
 {
 public:
@@ -130,6 +153,7 @@ KAboutLicense::Private::Private(LicenseKey licenseType, const KAboutData *aboutD
 
 KAboutLicense::Private::Private(const KAboutData *aboutData)
     : QSharedData(),
+      _licenseKey(Unknown),
       _aboutData(aboutData)
 {
 }
@@ -176,45 +200,49 @@ QString KAboutLicense::text() const
 {
     QString result;
 
-    const QString lineFeed = QString::fromLatin1("\n\n");
+    const QString lineFeed = QStringLiteral("\n\n");
 
     if (d->_aboutData && !d->_aboutData->copyrightStatement().isEmpty()) {
         result = d->_aboutData->copyrightStatement() + lineFeed;
     }
 
     bool knownLicense = false;
-    QString pathToFile;
+    QString pathToFile; // rel path if known license
     switch (d->_licenseKey) {
     case KAboutLicense::File:
         pathToFile = d->_pathToLicenseTextFile;
         break;
     case KAboutLicense::GPL_V2:
         knownLicense = true;
-        pathToFile = QStandardPaths::locate(QStandardPaths::GenericDataLocation, QString::fromLatin1("LICENSES/GPL_V2"));
+        pathToFile = QStringLiteral("GPL_V2");
         break;
     case KAboutLicense::LGPL_V2:
         knownLicense = true;
-        pathToFile = QStandardPaths::locate(QStandardPaths::GenericDataLocation, QString::fromLatin1("LICENSES/LGPL_V2"));
+        pathToFile = QStringLiteral("LGPL_V2");
         break;
     case KAboutLicense::BSDL:
         knownLicense = true;
-        pathToFile = QStandardPaths::locate(QStandardPaths::GenericDataLocation, QString::fromLatin1("LICENSES/BSD"));
+        pathToFile = QStringLiteral("BSD");
         break;
     case KAboutLicense::Artistic:
         knownLicense = true;
-        pathToFile = QStandardPaths::locate(QStandardPaths::GenericDataLocation, QString::fromLatin1("LICENSES/ARTISTIC"));
+        pathToFile = QStringLiteral("ARTISTIC");
         break;
     case KAboutLicense::QPL_V1_0:
         knownLicense = true;
-        pathToFile = QStandardPaths::locate(QStandardPaths::GenericDataLocation, QString::fromLatin1("LICENSES/QPL_V1.0"));
+        pathToFile = QStringLiteral("QPL_V1.0");
         break;
     case KAboutLicense::GPL_V3:
         knownLicense = true;
-        pathToFile = QStandardPaths::locate(QStandardPaths::GenericDataLocation, QString::fromLatin1("LICENSES/GPL_V3"));
+        pathToFile = QStringLiteral("GPL_V3");
         break;
     case KAboutLicense::LGPL_V3:
         knownLicense = true;
-        pathToFile = QStandardPaths::locate(QStandardPaths::GenericDataLocation, QString::fromLatin1("LICENSES/LGPL_V3"));
+        pathToFile = QStringLiteral("LGPL_V3");
+        break;
+    case KAboutLicense::LGPL_V2_1:
+        knownLicense = true;
+        pathToFile = QStringLiteral("LGPL_V21");
         break;
     case KAboutLicense::Custom:
         if (!d->_licenseText.isEmpty()) {
@@ -231,6 +259,8 @@ QString KAboutLicense::text() const
     }
 
     if (knownLicense) {
+        pathToFile = QStandardPaths::locate(QStandardPaths::GenericDataLocation,
+                QString::fromLatin1("kf5/licenses/") + pathToFile);
         result += QCoreApplication::translate(
                       "KAboutLicense",
                       "This program is distributed under the terms of the %1.").arg(name(KAboutLicense::ShortName));
@@ -284,6 +314,10 @@ QString KAboutLicense::name(KAboutLicense::NameFormat formatName) const
         licenseShort = QCoreApplication::translate("KAboutLicense", "LGPL v3", "@item license (short name)");
         licenseFull = QCoreApplication::translate("KAboutLicense", "GNU Lesser General Public License Version 3", "@item license");
         break;
+    case KAboutLicense::LGPL_V2_1:
+        licenseShort = QCoreApplication::translate("KAboutLicense", "LGPL v2.1", "@item license (short name)");
+        licenseFull = QCoreApplication::translate("KAboutLicense", "GNU Lesser General Public License Version 2.1", "@item license");
+        break;
     case KAboutLicense::Custom:
     case KAboutLicense::File:
         licenseShort = licenseFull = QCoreApplication::translate("KAboutLicense", "Custom", "@item license");
@@ -332,6 +366,8 @@ KAboutLicense KAboutLicense::byKeyword(const QString &rawKeyword)
         ldict.insert("gplv3+", KAboutLicense::GPL_V3);
         ldict.insert("lgplv3", KAboutLicense::LGPL_V3);
         ldict.insert("lgplv3+", KAboutLicense::LGPL_V3);
+        ldict.insert("lgplv21", KAboutLicense::LGPL_V2_1);
+        ldict.insert("lgplv21+", KAboutLicense::LGPL_V2_1);
     }
 
     // Normalize keyword.
@@ -359,9 +395,8 @@ public:
     QString _homepageAddress;
     QList<KAboutPerson> _authorList;
     QList<KAboutPerson> _creditList;
+    QList<KAboutPerson> _translatorList;
     QList<KAboutLicense> _licenseList;
-    QString translatorName;
-    QString translatorEmail;
     QString productName;
     QString programIconName;
     QVariant programLogo;
@@ -370,12 +405,16 @@ public:
 
     QString organizationDomain;
     QString _ocsProviderUrl;
+    QString desktopFileName;
 
     // Everything dr.konqi needs, we store as utf-8, so we
     // can just give it a pointer, w/o any allocations.
     QByteArray _internalProgramName;
     QByteArray _version;
     QByteArray _bugEmailAddress;
+
+    static QList<KAboutPerson> parseTranslators(const QString &translatorName, const QString &translatorEmail);
+
 };
 
 KAboutData::KAboutData(const QString &_componentName,
@@ -408,20 +447,27 @@ KAboutData::KAboutData(const QString &_componentName,
     d->_homepageAddress = homePageAddress;
     d->_bugEmailAddress = bugsEmailAddress.toUtf8();
 
-    if (d->_homepageAddress.contains(QLatin1String("http://"))) {
-        const int dot = d->_homepageAddress.indexOf(QLatin1Char('.'));
-        if (dot >= 0) {
-            d->organizationDomain = d->_homepageAddress.mid(dot + 1);
-            const int slash = d->organizationDomain.indexOf(QLatin1Char('/'));
-            if (slash >= 0) {
-                d->organizationDomain.truncate(slash);
-            }
-        } else {
-            d->organizationDomain = QString::fromLatin1("kde.org");
-        }
-    } else {
-        d->organizationDomain = QString::fromLatin1("kde.org");
+    QUrl homePageUrl(homePageAddress);
+    if (!homePageUrl.isValid() || homePageUrl.scheme().isEmpty()) {
+        // Default domain if nothing else is better
+        homePageUrl.setUrl(QStringLiteral("https://kde.org/"));
     }
+
+    const QChar dotChar(QLatin1Char('.'));
+    QStringList hostComponents = homePageUrl.host().split(dotChar);
+
+    // Remove leading component unless 2 (or less) components are present
+    if (hostComponents.size() > 2) {
+        hostComponents.removeFirst();
+    }
+
+    d->organizationDomain = hostComponents.join(dotChar);
+
+    // desktop file name is reverse domain name
+    std::reverse(hostComponents.begin(), hostComponents.end());
+    hostComponents.append(_componentName);
+
+    d->desktopFileName = hostComponents.join(dotChar);
 }
 
 KAboutData::KAboutData(const QString &_componentName,
@@ -445,7 +491,8 @@ KAboutData::KAboutData(const QString &_componentName,
     // match behaviour of other constructors
     d->_licenseList.append(KAboutLicense(KAboutLicense::Unknown, this));
     d->_bugEmailAddress = "submit@bugs.kde.org";
-    d->organizationDomain = QString::fromLatin1("kde.org");
+    d->organizationDomain = QStringLiteral("kde.org");
+    d->desktopFileName = QStringLiteral("org.kde.%1").arg(d->_componentName);
 }
 
 KAboutData::~KAboutData()
@@ -478,6 +525,19 @@ KAboutData &KAboutData::operator=(const KAboutData &other)
     return *this;
 }
 
+KAboutData KAboutData::fromPluginMetaData(const KPluginMetaData &plugin)
+{
+    KAboutData ret(plugin.pluginId(), plugin.name(), plugin.version(), plugin.description(),
+                   KAboutLicense::byKeyword(plugin.license()).key(), plugin.copyrightText(),
+                   plugin.extraInformation(), plugin.website());
+    ret.d->programIconName = plugin.iconName();
+    ret.d->_authorList = plugin.authors();
+    ret.d->_translatorList = plugin.translators();
+    ret.d->_creditList = plugin.otherContributors();
+    return ret;
+}
+
+
 KAboutData &KAboutData::addAuthor(const QString &name,
                                   const QString &task,
                                   const QString &emailAddress,
@@ -501,8 +561,7 @@ KAboutData &KAboutData::addCredit(const QString &name,
 KAboutData &KAboutData::setTranslator(const QString &name,
                                       const QString &emailAddress)
 {
-    d->translatorName = name;
-    d->translatorEmail = emailAddress;
+    d->_translatorList = Private::parseTranslators(name, emailAddress);
     return *this;
 }
 
@@ -744,48 +803,18 @@ QList<KAboutPerson> KAboutData::credits() const
     return d->_creditList;
 }
 
-#define NAME_OF_TRANSLATORS "Your names"
-#define EMAIL_OF_TRANSLATORS "Your emails"
-QList<KAboutPerson> KAboutData::translators() const
+QList<KAboutPerson> KAboutData::Private::parseTranslators(const QString &translatorName, const QString &translatorEmail)
 {
     QList<KAboutPerson> personList;
-#pragma message("KDE5 TODO: What about this code ?")
-#if 0
-    KLocale *tmpLocale = NULL;
-    if (KLocale::global()) {
-        // There could be many catalogs loaded into the global locale,
-        // e.g. in systemsettings. The tmp locale is needed to make sure we
-        // use the translators name from this aboutdata's catalog, rather than
-        // from any other loaded catalog.
-        tmpLocale = new KLocale(*KLocale::global());
-        tmpLocale->setActiveCatalog(catalogName());
-    }
-#endif
-    QString translatorName;
-    if (!d->translatorName.isEmpty()) {
-        translatorName = d->translatorName;
-    } else {
-        translatorName = QCoreApplication::translate("KAboutPerson", NAME_OF_TRANSLATORS, "NAME OF TRANSLATORS"); //toString(tmpLocale);
-    }
-
-    QString translatorEmail;
-    if (!d->translatorEmail.isEmpty()) {
-        translatorEmail = d->translatorEmail;
-    } else {
-        translatorEmail = QCoreApplication::translate("KAboutPerson", EMAIL_OF_TRANSLATORS, "EMAIL OF TRANSLATORS"); //toString(tmpLocale);
-    }
-#if 0
-    delete tmpLocale;
-#endif
-    if (translatorName.isEmpty() || translatorName == QString::fromUtf8(NAME_OF_TRANSLATORS)) {
+    if (translatorName.isEmpty() || translatorName == QStringLiteral("Your names")) {
         return personList;
     }
 
-    const QStringList nameList(translatorName.split(QString(QLatin1Char(','))));
+    const QStringList nameList(translatorName.split(QLatin1Char(',')));
 
     QStringList emailList;
-    if (!translatorEmail.isEmpty() && translatorEmail != QString::fromUtf8(EMAIL_OF_TRANSLATORS)) {
-        emailList = translatorEmail.split(QString(QLatin1Char(',')), QString::KeepEmptyParts);
+    if (!translatorEmail.isEmpty() && translatorEmail != QStringLiteral("Your emails")) {
+        emailList = translatorEmail.split(QLatin1Char(','), QString::KeepEmptyParts);
     }
 
     QStringList::const_iterator nit;
@@ -804,6 +833,12 @@ QList<KAboutPerson> KAboutData::translators() const
     return personList;
 }
 
+QList<KAboutPerson> KAboutData::translators() const
+{
+    return d->_translatorList;
+}
+
+
 QString KAboutData::aboutTranslationTeam()
 {
     return QCoreApplication::translate(
@@ -867,6 +902,18 @@ KAboutData &KAboutData::unsetCustomAuthorText()
     return *this;
 }
 
+KAboutData &KAboutData::setDesktopFileName(const QString &desktopFileName)
+{
+    d->desktopFileName = desktopFileName;
+
+    return *this;
+}
+
+QString KAboutData::desktopFileName() const
+{
+    return d->desktopFileName;
+}
+
 class KAboutDataRegistry
 {
 public:
@@ -882,18 +929,62 @@ public:
 
 Q_GLOBAL_STATIC(KAboutDataRegistry, s_registry)
 
+namespace {
+
+void warnIfOutOfSync(const char *aboutDataString, const QString &aboutDataValue,
+                     const char *appDataString, const QString &appDataValue)
+{
+    if (aboutDataValue != appDataValue) {
+        qCWarning(KABOUTDATA) << appDataString <<appDataValue << "is out-of-sync with" << aboutDataString << aboutDataValue;
+    }
+}
+
+}
+
 KAboutData KAboutData::applicationData()
 {
-    if (!s_registry->m_appData) {
-        s_registry->m_appData = new KAboutData(QCoreApplication::applicationName(),
-                                               QString(),
-                                               QString());
+    QCoreApplication *app = QCoreApplication::instance();
+
+    KAboutData *aboutData = s_registry->m_appData;
+
+    // not yet existing
+    if (!aboutData) {
+        // init from current Q*Application data
+        aboutData = new KAboutData(QCoreApplication::applicationName(),
+                                   QString(),
+                                   QString());
+        // For applicationDisplayName & desktopFileName, which are only properties of QGuiApplication,
+        // we have to try to get them via the property system, as the static getter methods are
+        // part of QtGui only. Disadvantage: requires an app instance.
+        // Either get all or none of the properties & warn about it
+        if (app) {
+            aboutData->setOrganizationDomain(QCoreApplication::organizationDomain().toUtf8());
+            aboutData->setVersion(QCoreApplication::applicationVersion().toUtf8());
+            aboutData->setDisplayName(app->property("applicationDisplayName").toString());
+            aboutData->setDesktopFileName(app->property("desktopFileName").toString());
+        } else {
+            qCWarning(KABOUTDATA) << "Could not initialize the properties of KAboutData::applicationData by the equivalent properties from Q*Application: no app instance (yet) existing.";
+        }
+
+        s_registry->m_appData = aboutData;
+    } else {
+        // check if in-sync with Q*Application metadata, as their setters could have been called
+        // after the last KAboutData::setApplicationData, with different values
+        warnIfOutOfSync("KAboutData::applicationData().componentName", aboutData->componentName(),
+                        "QCoreApplication::applicationName", QCoreApplication::applicationName());
+        warnIfOutOfSync("KAboutData::applicationData().version", aboutData->version(),
+                        "QCoreApplication::applicationVersion", QCoreApplication::applicationVersion());
+        warnIfOutOfSync("KAboutData::applicationData().organizationDomain", aboutData->organizationDomain(),
+                        "QCoreApplication::organizationDomain", QCoreApplication::organizationDomain());
+        if (app) {
+            warnIfOutOfSync("KAboutData::applicationData().displayName", aboutData->displayName(),
+                            "QGuiApplication::applicationDisplayName", app->property("applicationDisplayName").toString());
+            warnIfOutOfSync("KAboutData::applicationData().desktopFileName", aboutData->desktopFileName(),
+                            "QGuiApplication::desktopFileName", app->property("desktopFileName").toString());
+        }
     }
-    if (s_registry->m_appData->displayName().isEmpty() && QCoreApplication::instance()) {
-        // ## add a bool to avoid doing this too often?
-        s_registry->m_appData->setDisplayName(QCoreApplication::instance()->property("applicationDisplayName").toString());
-    }
-    return *s_registry->m_appData;
+
+    return *aboutData;
 }
 
 void KAboutData::setApplicationData(const KAboutData &aboutData)
@@ -904,13 +995,27 @@ void KAboutData::setApplicationData(const KAboutData &aboutData)
         s_registry->m_appData = new KAboutData(aboutData);
     }
 
+    // For applicationDisplayName & desktopFileName, which are only properties of QGuiApplication,
+    // we have to try to set them via the property system, as the static getter methods are
+    // part of QtGui only. Disadvantage: requires an app instance.
+    // So set either all or none of the properties & warn about it
     QCoreApplication *app = QCoreApplication::instance();
     if (app) {
         app->setApplicationVersion(aboutData.version());
         app->setApplicationName(aboutData.componentName());
         app->setOrganizationDomain(aboutData.organizationDomain());
         app->setProperty("applicationDisplayName", aboutData.displayName());
+        app->setProperty("desktopFileName", aboutData.desktopFileName());
+    } else {
+        qCWarning(KABOUTDATA) << "Could not initialize the equivalent properties of Q*Application: no instance (yet) existing.";
     }
+    // KF6: Rethink the current relation between KAboutData::applicationData and the Q*Application metadata
+    // Always overwriting the Q*Application metadata here, but not updating back the KAboutData
+    // in applicationData() is unbalanced and can result in out-of-sync data if the Q*Application
+    // setters have been called meanwhile
+    // Options are to remove the overlapping properties of KAboutData for cleancode, or making the
+    // overlapping properties official shadow properties of their Q*Application countparts, though
+    // that increases behavioural complexity a little.
 }
 
 void KAboutData::registerPluginData(const KAboutData &aboutData)
@@ -947,7 +1052,10 @@ bool KAboutData::setupCommandLine(QCommandLineParser *parser)
     }
 
     return parser->addOption(QCommandLineOption(QStringLiteral("author"), QCoreApplication::translate("KAboutData CLI", "Show author information.")))
-           && parser->addOption(QCommandLineOption(QStringLiteral("license"), QCoreApplication::translate("KAboutData CLI", "Show license information.")));
+           && parser->addOption(QCommandLineOption(QStringLiteral("license"), QCoreApplication::translate("KAboutData CLI", "Show license information.")))
+           && parser->addOption(QCommandLineOption(QStringLiteral("desktopfile"),
+                                                   QCoreApplication::translate("KAboutData CLI", "The base file name of the desktop entry for this application."),
+                                                   QCoreApplication::translate("KAboutData CLI", "file name")));
 }
 
 void KAboutData::processCommandLine(QCommandLineParser *parser)
@@ -958,7 +1066,7 @@ void KAboutData::processCommandLine(QCommandLineParser *parser)
         if (d->_authorList.isEmpty()) {
             printf("%s\n", qPrintable(QCoreApplication::translate("KAboutData CLI", "This application was written by somebody who wants to remain anonymous.")));
         } else {
-            printf(qPrintable(QCoreApplication::translate("KAboutData CLI", "%s was written by:\n")), qPrintable(qAppName()));
+            printf("%s\n", qPrintable(QCoreApplication::translate("KAboutData CLI", "%1 was written by:").arg(qAppName())));
             Q_FOREACH (const KAboutPerson &person, d->_authorList) {
                 QString authorData = QStringLiteral("    ") + person.name();
                 if (!person.emailAddress().isEmpty()) {
@@ -971,7 +1079,7 @@ void KAboutData::processCommandLine(QCommandLineParser *parser)
             if (bugAddress() == QLatin1String("submit@bugs.kde.org") ) {
                 printf("%s\n", qPrintable(QCoreApplication::translate("KAboutData CLI", "Please use http://bugs.kde.org to report bugs.")));
             } else if (!bugAddress().isEmpty()) {
-                printf(qPrintable(QCoreApplication::translate("KAboutData CLI", "Please report bugs to %s.\n")), qPrintable(bugAddress()));
+                printf("%s\n", qPrintable(QCoreApplication::translate("KAboutData CLI", "Please report bugs to %1.").arg(bugAddress())));
             }
         } else {
             printf("%s\n", qPrintable(customAuthorPlainText()));
@@ -983,6 +1091,11 @@ void KAboutData::processCommandLine(QCommandLineParser *parser)
         }
     }
 
+    const QString desktopFileName = parser->value(QStringLiteral("desktopfile"));
+    if (!desktopFileName.isEmpty()) {
+        d->desktopFileName = desktopFileName;
+    }
+
     if (foundArgument) {
         ::exit(EXIT_SUCCESS);
     }
